#! /bin/sh
#
# 	Part of kde-service-menu-pdf Version 0.5
# 	Copyright (C) 2011 Giuseppe Benigno <giuseppe.benigno(at)gmail.com>
#
# 	This program is free software: you can redistribute it and/or modify
# 	it under the terms of the GNU General Public License as published by
# 	the Free Software Foundation, either version 3 of the License, or
# 	(at your option) any later version.
#
# 	This program is distributed in the hope that it will be useful,
# 	but WITHOUT ANY WARRANTY; without even the implied warranty of
# 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# 	GNU General Public License for more details.
#
# 	You should have received a copy of the GNU General Public License
# 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

lang=`echo "${1}" | sed -e "s/@//g"`
shift
action="${1}"
shift
workDir="${1%/*}"; if [ ! -d "${workDir}" ]; then workDir="./"; fi
fileName="${1##*/}"
baseFileName="${fileName%.*}"
workFile="${@}"
originalTotalPages=`pdftk "${workFile}" dump_data | grep "NumberOfPages" | cut -d" " -f 2-`
[ $((originalTotalPages % 4)) -eq 0 ] && [ ${originalTotalPages} -le 20 ] && pagesPerBookletHint=${originalTotalPages} || pagesPerBookletHint=20

#### languages strings messages #################
# Syntax for strings name is: msg_[$action]_$window_[$section]
# For languages as sr@Latn use srLatn

load_language_en () {
	msg_pagesPerBooklet_title="Document \"${baseFileName}\""
	msg_pagesPerBooklet="Pages per booklet (suggest: $pagesPerBookletHint, total pages: $originalTotalPages)"
	msg_orientation_title="Orientation of document \"${baseFileName}\""
	msg_orientation_text="Select orientation of document \"${baseFileName}\""
	msg_orientation_leftToRight="The document is in left-to-right language"
	msg_orientation_rightToLeft="The document is in right-to-left language (such as Arabic or Hebrew)"
	msg_oneFile="Do you want all booklet in only one file?"
	msg_oneFile_outputFileName="${baseFileName} (booklet)"
	msg_saveFile_title="Save document"
	msg_progressbarDialog_title="Document \"${baseFileName}\""
	msg_progressbarDialog_text="Converting document \"${baseFileName}\" to booklet"
	msg_progressbarDialog_text2="Converting pages ... "
	msg_progressbarDialog_saving="Saving booklet N."
	msg_progressbarDialog_saving2="Saving booklet ..."
	msg_finish_title="Conversion of document \"${baseFileName}\""
	msg_finish="Done."
}

load_language_it () {
	msg_pagesPerBooklet_title="Documento \"${baseFileName}\""
	msg_pagesPerBooklet="Pagine per opuscolo (suggerite: $pagesPerBookletHint, pagine totali: $originalTotalPages)"
	msg_orientation_title="Orientazione del documento \"${baseFileName}\""
	msg_orientation_text="Seleziona l'orientazione del documento \"${baseFileName}\""
	msg_orientation_leftToRight="Lingua da sinistra a destra (come inglese o italiano)"
	msg_orientation_rightToLeft="Lingua da destra a sinistra (come arabo o ebraico)"
	msg_oneFile="Vuoi salvare gli opuscoli in un solo file?"
	msg_oneFile_outputFileName="${baseFileName} (opuscolo)"
	msg_saveFile_title="Salva documento"
	msg_progressbarDialog_title="Documento \"${baseFileName}\""
	msg_progressbarDialog_text="Conversione del documento \"${baseFileName}\" in opuscolo"
	msg_progressbarDialog_text2="Conversione delle pagine ..."
	msg_progressbarDialog_saving="Salvataggio dell'opuscolo N."
	msg_progressbarDialog_saving2="Salvataggio dell'opuscolo ..."
	msg_finish_title="Conversione del documento \"${baseFileName}\""
	msg_finish="Completata."
}

load_language_de () {
	msg_pagesPerBooklet_title="Dokument \"${baseFileName}\""
	msg_pagesPerBooklet="Seiten pro Booklet (vorschlagen: $pagesPerBookletHint, Gesamte Seiten: $originalTotalPages)"
	msg_orientation_title="Dokument \"${baseFileName}\""
	msg_orientation_text="Ausrichtung des Dokuments \"${baseFileName}\" auswählen"
	msg_orientation_leftToRight="Das Dokument liest sich von links nach rechts"
	msg_orientation_rightToLeft="Das Dokument liest sich von rechts nach links (z.B. Hebräisch oder Arabisch)"
	msg_oneFile="Wollen Sie alle Booklets in einer Datei?"
	msg_oneFile_outputFileName="${baseFileName} (Booklet)"
	msg_saveFile_title="Dokument speichern"
	msg_progressbarDialog_title="Dokument \"${baseFileName}\""
	msg_progressbarDialog_text="Wandle Dokument \"${baseFileName}\" in Booklet um"
	msg_progressbarDialog_text2="Seiten werden umgewandelt ... "
	msg_progressbarDialog_saving="Speichere Boocklet Nr."
	msg_progressbarDialog_saving2="Speichere Booklet ..."
	msg_finish_title="Umwandlung von \"${baseFileName}\""
	msg_finish="Fertig."
}
################################################

all () {
	pagesPerBooklet=`kdialog --caption "${msg_pagesPerBooklet_title}" --icon configure --combobox "${msg_pagesPerBooklet}" $(seq 4 4 $((originalTotalPages*2+2)))` || exit
	[ "$pagesPerBooklet" ] || pagesPerBooklet=$pagesPerBookletHint

	case "`kdialog --caption "${msg_orientation_title}" --icon format-text-direction-ltr --radiolist "${msg_orientation_text}" 1 "${msg_orientation_leftToRight}" on 0 "${msg_orientation_rightToLeft}" off --geometry 350x150`" in
		0) rightToLeft="TRUE";;
		1) rightToLeft="FALSE";;
		*) exit 0;;
	esac

	## calculate empty pages to add and final total pages
	[ $originalTotalPages -eq $pagesPerBooklet ] && empty_pages_to_add=0 || empty_pages_to_add=$((pagesPerBooklet - (originalTotalPages % pagesPerBooklet)))
	totalPages=$((originalTotalPages + empty_pages_to_add))

	kdialog --yesno "${msg_oneFile}"
	case "$?" in
		0) oneFile="TRUE";
			outputFileName=`kdialog --title "${msg_saveFile_title}" --caption "${msg_saveFile_title}" --icon document-save --getsavefilename "${workDir}/${msg_oneFile_outputFileName}.pdf" application/pdf` || exit;;
		1) oneFile="FALSE";;
		*) exit 0;;
	esac

	n='0'; startPage='0'; string=''
	progressbarDialog=`kdialog -title "${msg_progressbarDialog_title}" --icon kjournal --progressbar "${msg_progressbarDialog_text}" $totalPages`
	##qdbus $progressbarDialog org.kde.kdialog.ProgressDialog.showCancelButton true
	##until test "true" = `qdbus $progressbarDialog wasCancelled`; do
		while [ "$(($startPage+$pagesPerBooklet))" -le "$totalPages" ]; do
			a=$((startPage+1))
			b=$((startPage+pagesPerBooklet))
			while [ "$a" -lt "$b" ]; do
				[ "$a" -gt "${originalTotalPages}" ] && aa='{}' && aa1='{}' || { aa=$a; aa1=$((a+1)); }
				[ "$b" -gt "${originalTotalPages}" ] && bb='{}' && bb1='{}' || { bb=$b; bb1=$((b-1)); }
				[ x"$rightToLeft" = x'FALSE' ] && string="${string},${bb},${aa},${aa1},${bb1}" || string="${string},${aa},${bb},${bb1},${aa1}"
				a=$((a+2))
				b=$((b-2))
				qdbus $progressbarDialog org.kde.kdialog.ProgressDialog.setLabelText "${msg_progressbarDialog_saving} $(($startPage+1))-$(($startPage+$pagesPerBooklet))"
				qdbus $progressbarDialog Set org.kde.kdialog.ProgressDialog value $a
			done

			if [ x"$oneFile" = x'FALSE' ];
				then qdbus $progressbarDialog org.kde.kdialog.ProgressDialog.setLabelText "${msg_progressbarDialog_saving} ${n} ..."
					n=$((n+1))
					pdfnup --outfile "${outputFileName}" -- "${workFile}" "${string#,}"
					string=''
					sleep 1
			fi
			startPage=$(($startPage+$pagesPerBooklet))
		done
		[ x"$oneFile" = x'TRUE' ] && qdbus $progressbarDialog org.kde.kdialog.ProgressDialog.setLabelText "${msg_progressbarDialog_saving2}" && pdfnup --outfile "${outputFileName}" -- "${workFile}" "${string#,}"
	##done
	qdbus $progressbarDialog org.kde.kdialog.ProgressDialog.close
	kdialog --title "${msg_finish_title}" --passivepopup "${msg_finish}" 5
}

#### main

cd "${workDir}"
    "load_language_${lang}" || load_language_en
	"${action}" "${@}"
cd -
